/******************************************************************************/
/*Files to Include                                                            */
/******************************************************************************/

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */

#include "user.h"

/******************************************************************************/
/* Interrupt Routines                                                         */
/******************************************************************************/

/* Baseline devices don't have interrupts. Note that some PIC16's 
 * are baseline devices.  Unfortunately the baseline detection macro is 
 * _PIC12 */
#ifndef _PIC12

void interrupt isr(void) {


    if (PIR1bits.SSP1IF) {
        if (SSP1STATbits.BF) {
            /* Store recieved byte to SPI buffer */
            DATA_BUFFER[--BYTES_COUNT] = SSPBUF;


            if (BYTES_COUNT == 0) {
                CLC_ON;
                CS_OUT = 0;

                SSPBUF = DATA_BUFFER[23];
                BYTES_COUNT = 24;
            }                /* Load data to be send on SDO */
            else {
                SSPBUF = DATA_BUFFER[BYTES_COUNT - 1];
            }

        }
        /* Clear interrupt flag */
        PIR1bits.SSP1IF = 0;
    }
    else if (INTCONbits.IOCIF) {
        
        if (!IOCAPbits.IOCAP4) {


            /* Falling edge detected */
            IOCAPbits.IOCAP4 = 1;
            IOCANbits.IOCAN4 = 0;
            IOCAFbits.IOCAF4 = 0;

            /* Reset some variables */
            BYTES_COUNT = 24;

            SSPBUF = 0;

        } else {
            TEST = 0;
            /* Rise SS to idle state */
            CS_OUT = 1;

            CLC_OFF;

            /* Rising edge detected */
            IOCAPbits.IOCAP4 = 0;
            IOCANbits.IOCAN4 = 1;
            IOCAFbits.IOCAF4 = 0;

            /* Latch DATA_BUFFER into VIDEO_BUFFER */
            /* ROW1 dd*/
            VIDEO_BUFFER[1] = DATA_BUFFER[2];
            VIDEO_BUFFER[2] = DATA_BUFFER[1];
            VIDEO_BUFFER[3] = DATA_BUFFER[0];

            /* ROW2... */
            VIDEO_BUFFER[5] = DATA_BUFFER[5];
            VIDEO_BUFFER[6] = DATA_BUFFER[4];
            VIDEO_BUFFER[7] = DATA_BUFFER[3];

            /* ROW3 */
            VIDEO_BUFFER[9] = DATA_BUFFER[8];
            VIDEO_BUFFER[10] = DATA_BUFFER[7];
            VIDEO_BUFFER[11] = DATA_BUFFER[6];

            /* ROW4 */
            VIDEO_BUFFER[13] = DATA_BUFFER[11];
            VIDEO_BUFFER[14] = DATA_BUFFER[10];
            VIDEO_BUFFER[15] = DATA_BUFFER[9];

            /* ROW5 */
            VIDEO_BUFFER[17] = DATA_BUFFER[14];
            VIDEO_BUFFER[18] = DATA_BUFFER[13];
            VIDEO_BUFFER[19] = DATA_BUFFER[12];

            /* ROW6 */
            VIDEO_BUFFER[21] = DATA_BUFFER[17];
            VIDEO_BUFFER[22] = DATA_BUFFER[16];
            VIDEO_BUFFER[23] = DATA_BUFFER[15];

            /* ROW7 */
            VIDEO_BUFFER[25] = DATA_BUFFER[20];
            VIDEO_BUFFER[26] = DATA_BUFFER[19];
            VIDEO_BUFFER[27] = DATA_BUFFER[18];

            /* ROW8 */
            VIDEO_BUFFER[29] = DATA_BUFFER[23];
            VIDEO_BUFFER[30] = DATA_BUFFER[22];
            VIDEO_BUFFER[31] = DATA_BUFFER[21];


        }
        /* Clear interrupt flag */
        INTCONbits.IOCIF = 0;
        TEST = 1;
    }


}
#endif


