/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */

#include "user.h"
/******************************************************************************/
/* User Functions                                                             */

/******************************************************************************/

void InitGPIO(void) {
    ANSELA = 0;
    ANSELC = 0;

    TRISA = 0x00;
    LATA = 0x00;
    TRISC = 0x00;
    LATC = 0x00;



    /* Initialize peripheral SPI */
#ifdef _16F1823
    TRISCbits.TRISC3 = 1; //SS
#else
    TRISAbits.TRISA3 = 1; //SS
#endif
    TRISAbits.TRISA4 = 1; //SS
    TRISCbits.TRISC0 = 1; //SCK
    TRISAbits.TRISA5 = 1; //SCK
    TRISCbits.TRISC2 = 0; //SDO
    TRISCbits.TRISC1 = 1; //SDI

    TRISCbits.TRISC4 = 0;
    CS_OUT = 1;

    /* Initialize serial SPI */
    TRISAbits.TRISA0 = 0; //SCK
    TRISAbits.TRISA1 = 0; //SS
    TRISAbits.TRISA2 = 0; //SDO

    /* Initialize CLC */
#ifdef _16F1503
    TRISAbits.TRISA5 = 1; //CLC1IN1
    TRISCbits.TRISC5 = 0; //CLC1
    LATCbits.LATC5 = 0;
#endif


    APFCONbits.SDOSEL = 0; //SDO on RC2
#ifdef _16F1823
    APFCONbits.SSSEL = 0; //SS on RC3
#else
    APFCONbits.SSSEL = 1; //SS on RA3
    APFCONbits.CLC1SEL = 1; //CLC1 on RC5
#endif

    TRISCbits.TRISC3 = 0;
    TEST = 1;

}

void InitSPI(void) {
    SSP1STATbits.CKE = 0; //Transition on falling edge
    SSP1STATbits.SMP = 0; //Sampling in the middle
    SSP1CON1bits.SSPM = 4; //SPI slave + SS control
    SSP1CON1bits.CKP = 0; // Clock polarity

}

void InitVIDEO(void) {
    VIDEO_BUFFER[0] = 0x80;
    VIDEO_BUFFER[4] = 0x40;
    VIDEO_BUFFER[8] = 0x20;
    VIDEO_BUFFER[12] = 0x10;
    VIDEO_BUFFER[16] = 0x08;
    VIDEO_BUFFER[20] = 0x04;
    VIDEO_BUFFER[24] = 0x02;
    VIDEO_BUFFER[28] = 0x01;
}

void InitInterrupt(void) {

    /* Wait SS to be IDLE */
#ifdef _16F1823
    while (!PORTCbits.RC3);
#else
    while (!PORTAbits.RA3);
#endif
    SSP1CON1bits.SSPEN = 1;

    /* Enable IOC -> SCK */
    IOCAFbits.IOCAF4 = 0; //Clear flag
    IOCANbits.IOCAN4 = 1; //Falling edge detection -> Enable
    IOCAFbits.IOCAF4 = 0; //Rising edge detection -> Disable

    INTCONbits.IOCIF = 0;
    INTCONbits.IOCIE = 1;
    PIR1bits.SSP1IF = 0;
    PIE1bits.SSP1IE = 1;
    INTCONbits.PEIE = 1;
    GIE = 1;
}

void InitCLC(void) {
    NCO1CLKbits.N1CKS = 11;
    NCO1CONbits.N1EN = 1;

    // Generated by CLC Designer, Version: 3.0.0.2
    // Date: 10/04/2013 08:45
    // Device:PIC16(L)F1503

    CLC1GLS0 = 0x02;
    CLC1GLS1 = 0x02;
    CLC1GLS2 = 0x02;
    CLC1GLS3 = 0x02;
    CLC1SEL0 = 0x01;
    CLC1SEL1 = 0x00;
    CLC1POL = 0x00;
    CLC1CON = 0x82;
}



void InitApp(void) {
    InitGPIO();
    InitVIDEO();
    InitSPI();
#ifdef _16F1503
    InitCLC();
#endif
    InitInterrupt();


}

void DelayUs(unsigned int us) {
    while (us--) {
        asm("NOP");
        asm("NOP");
        if (us == 0)
            break;
    }
}

